#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <netdb.h> 
#include <netinet/in.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

void handler(int signo)
{
        int status;
        /* gestisco tutti i figli terminati */
        while (waitpid(-1, &status, WNOHANG) > 0);
}

int main(int argc, char **argv)
{
        int sd, err, on;
        struct addrinfo hints, *res;

        signal(SIGCHLD, handler);

        memset(&hints, 0, sizeof(hints));
        /* Usa AF_INET per forzare solo IPv4, AF_INET6 per forzare solo IPv6 */
        hints.ai_family = AF_UNSPEC;
        hints.ai_socktype = SOCK_STREAM;
        hints.ai_flags = AI_PASSIVE;

        if ((err = getaddrinfo(NULL, argv[1], &hints, &res)) != 0) {
                fprintf(stderr, "Errore setup indirizzo bind: %s\n", gai_strerror(err));
                exit(EXIT_FAILURE);
        }

        if ((sd = socket(res->ai_family, res->ai_socktype, res->ai_protocol)) < 0){
                perror("Errore in socket");
                exit(EXIT_FAILURE);
        }

        on = 1;
        if(setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on))<0){
                perror("setsockopt");
                exit(EXIT_FAILURE);
        }

        if (bind(sd, res->ai_addr, res->ai_addrlen) < 0) {
                perror("Errore in bind");
                exit(EXIT_FAILURE);
        }

        freeaddrinfo(res);

        /* trasforma in socket passiva d'ascolto */
        listen(sd, SOMAXCONN);

        for(;;) {
                int ns, pid, nread;
                ns = accept(sd, NULL, NULL);

                if ((pid = fork()) < 0) {
                        perror("fork");
                        exit(EXIT_FAILURE);
                } else if (pid == 0) { /* FIGLIO */
                        char str1[4096], str2[4096], response[80];
						int i;

                        /* Chiudo la socket passiva */
                        close(sd);

                        /* Leggo un byte alla volta fino al terminatore. */
                        i = 0;
						do{
							if ( (nread = read(ns, str1+i, 1) ) < 0) {
									perror("read");
									exit(EXIT_FAILURE);
							}
							i++;
						}
						while( str1[i-1] != '\0' );

                        /* Leggo un byte alla volta fino al terminatore. */
                        i = 0;
						do{
							if ( (nread = read(ns, str2+i, 1) ) < 0) {
									perror("read");
									exit(EXIT_FAILURE);
							}
							i++;
						}
						while( str2[i-1] != '\0' );

                        /* Preparo il buffer contenente la risposta */
                        if (strcmp(str1, str2) == 0) {
                                /* Le stringhe sono identiche */
                                strcpy(response, "SI");
                        } else {
                                /* Le stringhe sono diverse */
                                strcpy(response, "NO");
                        }

                        /* Invio la risposta */
                        write(ns, response, strlen(response));

                        /* Chiudo la socket attiva */
                        close(ns);

                        /* Termino il figlio */
                        exit(EXIT_SUCCESS);
                }

                /* PADRE */
                
                /* Chiudo la socket attiva */
                close(ns);
        }

        return 0;
}

